"use client";

import { Environment, Html, OrbitControls, useProgress } from "@react-three/drei";
import { Canvas, useThree } from "@react-three/fiber";
import gsap from "gsap";
import React, { useEffect, useRef } from "react";
import { Camera, PerspectiveCamera, Vector3 } from "three";
import Model from "./model";

type selectType = {
	pool: number,
	floor: number,
	ladder: number,
	waterfall: number,
	jacuzzi: number,
	lamp: number,
	showWater: boolean,
	selectedTile?: {
		id: number,
		texture: string,
		name: string,
		model_3d?: string
	} | null,
	selectedLadder?: {
		id: number,
		texture: string,
		name: string,
		model_3d?: string
	} | null
}
type cameraSetting = { position: { x: number, y: number, z: number }, target: { x: number, y: number, z: number } }

function Loader() {
	const { progress } = useProgress();
	return <Html center>{progress.toFixed(1)}% loaded</Html>;
}

const Scene = ({ showItem, cameraSetting }: { showItem: selectType; cameraSetting?: cameraSetting }) => {
	const controlsRef = useRef<any>(null); // Ref for OrbitControls

	return (
		<Canvas style={{ position: "relative", height: "100vh", width: "100%", background: "transparent" }}>
			<React.Suspense fallback={<Loader />}>
				<SetCameraPosition cameraSetting={cameraSetting} controlsRef={controlsRef} />
				<OrbitControls ref={controlsRef} />
				<Model showItem={showItem} />
				<Environment preset="apartment" />
			</React.Suspense>
		</Canvas>
	);
};

const animateCamera = (camera: Camera, controls: any, newPosition: Vector3, newTarget: Vector3, duration = 2) => {
	if (!(camera instanceof PerspectiveCamera)) {
		console.error("Camera is not a PerspectiveCamera");
		return;
	}

	// Animate camera position
	gsap.to(camera.position, {
		x: newPosition.x,
		y: newPosition.y,
		z: newPosition.z,
		duration,
		onUpdate: () => camera.updateProjectionMatrix(),
	});

	// Animate OrbitControls target
	gsap.to(controls.target, {
		x: newTarget.x,
		y: newTarget.y,
		z: newTarget.z,
		duration,
		onUpdate: () => controls.update(), // Make sure to call update() on controls
	});
};

const SetCameraPosition = ({
	cameraSetting,
	controlsRef,
}: {
	cameraSetting?: cameraSetting;
	controlsRef: React.RefObject<any>;
}) => {
	const { camera } = useThree();

	useEffect(() => {
		const handleLogPosition = () => {
			console.log("Camera Position:", camera.position);
			const lookAt = new Vector3();
			camera.getWorldDirection(lookAt);
			console.log("Camera LookAt Direction:", lookAt);
			console.log("Camera Zoom:", camera.zoom);
		};

		window.addEventListener("keydown", (event) => {
			if (event.key === "l") {
				handleLogPosition();
			}
		});

		return () => {
			window.removeEventListener("keydown", handleLogPosition);
		};
	}, [camera]);

	useEffect(() => {
		if (cameraSetting && controlsRef.current) {
			const { position, target } = cameraSetting;
			const newPosition = new Vector3(position.x, position.y, position.z);
			const newTarget = new Vector3(target.x, target.y, target.z);
			animateCamera(camera, controlsRef.current, newPosition, newTarget);
		} else if (controlsRef.current) {
			const defaultPosition = new Vector3(2.495087891261038, 1.3182932241842755, 0.6962728967548462);
			const defaultTarget = new Vector3(-0.7679973185038164, -0.4873719997057038, -0.41551011139779925);
			animateCamera(camera, controlsRef.current, defaultPosition, defaultTarget);
		}
	}, [camera, cameraSetting, controlsRef]);

	return null;
};

export default Scene;
